package com.dsk.flink.dsc.common.function;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.flink.api.common.functions.MapFunction;
import org.apache.flink.api.java.tuple.Tuple3;

import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

public class CanalMapToTsGroupFunction implements MapFunction<JSONObject, Tuple3<JSONObject, String, Long>> {
    @Override
    public Tuple3<JSONObject, String, Long> map(JSONObject value) throws Exception {

        JSONArray dataList = value.getJSONArray("data");
        JSONObject mysqlType = value.getJSONObject("mysqlType");
        String table = value.getString("table");
        JSONArray pkNames = value.getJSONArray("pkNames");
        Set<String> pkNameSet = new HashSet<>();
        long ts = value.getLong("ts") == null ? System.currentTimeMillis() : value.getLong("ts");
        if (CollUtil.isNotEmpty(pkNames)) {
            pkNames.forEach(name -> pkNameSet.add(String.valueOf(name)));
        }
        JSONObject dataObj = dataList.getJSONObject(0);
        String groupKey = table;
        for (String pk : pkNameSet) {
            String pkValue = getValueString(dataObj, pk,
                    mysqlType.getString(pk));
            groupKey = table.concat("-").concat(pkValue);
        }

        return Tuple3.of(value, groupKey, ts);
    }

    private static final String[] STR_SQL_TYPE = new String[]{"VARCHAR","CHAR","TINYBLOB","BLOB","MEDIUMBLOB","LONGBLOB","TINYTEXT","TEXT","MEDIUMTEXT","LONGTEXT","TIME","TIMESTAMP","JSON","json"};


    /**
     * @author shezaixing
     * @date 2023/12/7 14:23
     * @description 判断拼接字符串时类型（是否需要加上引号）
     *
     */
    private static String getValueString(JSONObject dataObj,String columnKey,String mysqlType){

        if(null == dataObj.get(columnKey)){
            return "null";
        }

        //需要处理成字符串加引号的类型
        if(Arrays.asList(STR_SQL_TYPE).contains(mysqlType.toUpperCase())){
            return String.format("'%s'", dataObj.getString(columnKey).replace("\\","\\\\").replace("'", "\\'") );
        }

        //时间字段处理
        if("DATE".equalsIgnoreCase(mysqlType) || "DATETIME".equalsIgnoreCase(mysqlType)){
            SimpleDateFormat df = "DATETIME".equalsIgnoreCase(mysqlType) ? new SimpleDateFormat("yyyy-MM-dd HH:mm:ss") : new SimpleDateFormat("yyyy-MM-dd");
            return String.format("\"%s\"",df.format(dataObj.getDate(columnKey)));
        }

        return dataObj.getString(columnKey);
    }
}
